pub struct Gc<'b, T: ?Sized>(/* private fields */);Expand description
A thin, copyable, garbage collected pointer type.
Implementations
Sourceimpl<'b, T: ?Sized> Gc<'b, T>
impl<'b, T: ?Sized> Gc<'b, T>
pub fn write(&self) -> &Write<T>
pub fn unlock(&self) -> &T::Unlockedwhere
T: Unlock,
pub fn as_ptr(&self) -> *mut T
Sourcepub fn long_ref(&self) -> &'b T
pub fn long_ref(&self) -> &'b T
Returns a reference to the contained value, which lasts for as long as the current view is active.
pub fn downgrade(this: Gc<'b, T>) -> Weak<'b, T>